#include <math.h>
#include "geoggeoccnvc.h"
 
/*
     online tool at http://www.apsalin.com/convert-geodetic-to-cartesian.aspx
     and then look to xyz to get desired geocentric angle
 
     Geodetic Coordinate Conversions
     James R. Clynch
     Naval Postgraduate School, 2002
     www.oc.nps.edu/oc2902w/coord/coordcvt.pdf
 
     https://en.wikipedia.org/wiki/Latitude
     f = flattening = (a-b)/b where b = polar radius and a = equatorial radius
     e^2 = 2f - f^2
     b = a(1-f) = a(sqrt(1-e^2)
     WGS84 ellipsoid
     a (equatorial radius): 6378137.0 m exactly
     1/f =  (inverse flattening): 298.257223563 exactly
       from which
     b (polar radius): 6356752.3142 m
     e^2 (eccentricity squared): 0.00669437999014
     tan lat_geocentric = (1 - e^2) tan lat_geodetic
 

     From PROGRAMS.330/VOLI/src/udelaz.c
*/
static double degrad = 0.0174532925199433;     /* degrees to radians */
static double a = 6378.137 ;                   /* equatorial radius */
static double b = 6356.752 ;                   /* polar radius = a(1-f) */
static double f = 0.003352810664747481 ;       /* f = 1 - b/a = 
                                                    1/298.257223563 */
static double e = 0.081819191 ;                        /* eccentricity (A.38)
                                                   e^2 = (a^2 - b^2)/a^2
                                                   e^2 = 2f - f^2 */
static double e2 = 0.006694379990141317 ;      /* e^2 */
static double eps = 0.006739496742276435 ;     /* second ellipticity (A.42)
                                                   eps = (a^2 - b^2)/b^2
                                                   = e^2/(1-e^2) */
static float RE=6371.003;
 
void geog_to_geoc(double geoglat,double geoglon,double *geoclat,double *geoclon){
double c, s, clat, slat, fac, e4;
/*
      longitude does not change
*/
      *geoclon = geoglon ;
/*
      the conversion formula is typically given in terms of tan
      by forming cos and sin and normalizing so that cos^2 + sin^2 = 1
      we can use the atan2 to get proper angle in radians
*/
      c = cos(degrad*geoglat) ;
      s = sin(degrad*geoglat) ;
      e4 = (1.-e2)*(1.-e2) ;
      fac = sqrt(e4 + (1.0-e4)*c*c) ;
      slat = (1.-e2) * s /fac ;
      clat =           c /fac ;
      *geoclat = atan2(slat,clat)/degrad ;
}
      

void geoc_to_geog(double geoclat,double geoclon,double *geoglat,double *geoglon){
double c, s, clat, slat, fac, e4;
/*
      longitude does not change
*/
      *geoglon = geoclon ;
/*
      the conversion formula is typically given in terms of tan
      by forming cos and sin and normalizing so that cos^2 + sin^2 = 1
      we can use the atan2 to get proper angle in radians
*/
      c = cos(degrad*geoclat) ;
      s = sin(degrad*geoclat) ;
      e4 = (1.-e2)*(1.-e2) ;
      fac = sqrt( s*s/e4 + c*c) ;
      slat =           s /(fac*(1-e2)) ;
      clat =           c /(fac) ;
      *geoglat = atan2(slat,clat)/degrad ;
}
